<?php

namespace FiloBlu\GcpLogging\Logger\Handler;

use FiloBlu\GcpLogging\Helper\Data;
use Google\Cloud\Logging\Logger;
use Google\Cloud\Logging\LoggingClient;
use Magento\Framework\Logger\Handler\Base as BaseHandler;

class Handler extends BaseHandler
{
    /**
     * @var LoggerInterface[]
     */
    protected $loggers;

    /**
     * @var Data
     */
    protected $data;

    /**
     * Handler constructor.
     *
     * @param Data $data
     * @param bool|int $level
     * @param bool $bubble
     */
    public function __construct(Data $data, $level = Logger::DEBUG, $bubble = true)
    {
        $this->data = $data;
        $this->level = $level;
        $this->bubble = $bubble;
    }

    /**
     * {@inheritdoc}
     */
    public function handle(array $record)
    {
        if (!$this->data->gcpLoggingIsEnabled()) {
            return false;
        }

        if (!$this->isHandling($record)) {
            return false;
        }

        $contextData = [
            'client_log' => $record['context']
        ];

        $contextData['module'] = $this->data->moduleName();
        $contextData['site'] = $this->data->getBackEndUrl();
        $contextData['stack_trace'] = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 20);

        $record['context'] = $contextData;

        if ($this->data->getGcpLoggingType() == Data::LOG_TYPE_GCP) {
            $this->getLogger($record['channel'])->log(strtolower($record['level_name']), $record['message'], $record['context']);

            return false === $this->bubble;
        } else {
            // TODO: manage log in file
            return false;
        }
    }

    /**
     * @param $channel
     *
     * @return LoggerInterface
     */
    protected function getLogger($channel)
    {
        $client = new LoggingClient(
            [
                'projectId' => $this->data->getProjectId(),
                'keyFile' => json_decode($this->data->getKey(), true)
            ]
        );

        if (!isset($this->loggers[$channel])) {
            $this->loggers[$channel] = $client->psrLogger($this->data->getLogFileName(), ['labels' => ['context' => $channel]]);
        }

        return $this->loggers[$channel];
    }
}
